
#pragma optimize("",off)
#pragma comment(linker,"/FILEALIGN:512 /MERGE:.rdata=.text /MERGE:.data=.text /SECTION:.text,EWR /IGNORE:4078")

#include <windows.h>
#include <stdio.h>
#include <imagehlp.h>
#include "realign.h"

// constants
const PSTR  PROC_TITLE      = "----------------------------------\n"\
						      " yoda's PE Realigner 2.0\n"\
						      "----------------------------------\n";
const PSTR  CMD_OPTIONS     = " Usage: yPER.exe [filename] [OPTIONS]\n"
                              " Options:\n"
							  " /REALIGN_NORMAL\n"
							  " /REALIGN_NICE\n"
							  " /REALIGN_HARDCORE\n"
							  " /WIPERELOC\n"
							  " /VALIDATE";
const char *szRealign0Cmd   = "/REALIGN_NORMAL";
const char *szRealign1Cmd   = "/REALIGN_NICE";
const char *szRealign2Cmd   = "/REALIGN_HARDCORE";
const char *szWipeRelocCmd  = "/WIPERELOC";
const char *szValidateCmd   = "/VALIDATE";
const char *szOK            = "Done\n";
const char *szErr           = "error :(\n";

int main(int argc, char* argv[])
{
	HANDLE   hFile,hMap;
	LPVOID   pMap;
	char     cPath[MAX_PATH];
	DWORD    dwFsize, dwNewFsize, dwRet;
	UINT     i;

	// check the commandline
	printf(PROC_TITLE);
	if (argc < 3)
	{
		printf(" Not enough parameters !\n");
		printf(CMD_OPTIONS);
		return -1;
	}
	// get path
	lstrcpy(cPath, argv[1]);

	// map the target file !
	hFile = CreateFile (cPath, GENERIC_READ | GENERIC_WRITE, \
	FILE_SHARE_READ | FILE_SHARE_WRITE,	NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
	if (hFile == INVALID_HANDLE_VALUE)
	{
		printf(" File IO-Error :(\n");
		return -1;
	}
	dwFsize = GetFileSize(hFile,0);
	hMap = CreateFileMapping (hFile, NULL, PAGE_READWRITE, 0, 0,NULL);
	CloseHandle(hFile);
	if (!hMap)
	{
		printf(" Error while mapping file :(\n");
		return -1;
	}
	pMap = MapViewOfFile ( hMap, FILE_MAP_READ | FILE_MAP_WRITE, 0, 0, 0);
	CloseHandle(hMap);
	if (!pMap)
	{
		printf(" Error while mapping file :(\n");
		return -1;
	}

	// PE file ?
	if (!ImageNtHeader(pMap))
	{
		printf("Invalid PE file !\n");
		return -1;
	}

	// do the operations
	dwNewFsize = dwFsize;
	for (i=2; i < (UINT)argc; i++)
	{
		if (lstrcmpi(argv[i], szRealign0Cmd) == 0)
		{
			printf("->Normal realign...");
			dwRet = RealignPE(pMap, dwNewFsize, 0);
			if (REALIGNDLLAPI_SUCCESS(dwRet))
			{
				printf(szOK);
				dwNewFsize = dwRet;
			}
			else
				printf(szErr);			
		}
		else if (lstrcmpi(argv[i], szRealign1Cmd) == 0)
		{
			printf("->Nice realign...");
			dwRet = RealignPE(pMap, dwNewFsize, 2);
			if (REALIGNDLLAPI_SUCCESS(dwRet))
			{
				printf(szOK);
				dwNewFsize = dwRet;
			}
			else
				printf(szErr);
		}
		else if (lstrcmpi(argv[i], szRealign2Cmd) == 0)
		{
			printf("->Harcore realign...");
			dwRet = RealignPE(pMap, dwNewFsize, 1);
			if (REALIGNDLLAPI_SUCCESS(dwRet))
			{
				printf(szOK);
				dwNewFsize = dwRet;
			}
			else
				printf(szErr);
		}
		else if (lstrcmpi(argv[i], szWipeRelocCmd) == 0)
		{
			printf("->Wipe relocation...");
			dwRet = WipeReloc(pMap, dwNewFsize);
			if (REALIGNDLLAPI_SUCCESS(dwRet))
			{
				printf(szOK);
				dwNewFsize = dwRet;
			}
			else
				printf(szErr);
		}
		else if (lstrcmpi(argv[i], szValidateCmd) == 0)
		{
			printf("->Validate image...");
			if (ValidatePE(pMap, dwNewFsize))
				printf(szOK);
			else
				printf(szErr);
		}
		else
			printf("Invalid Option: \"%s\"\n", argv[i]);
	}

	// unmap
	UnmapViewOfFile(pMap);
	Sleep(200);

	// force the new filesize
	hFile = CreateFile (cPath,
		GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		0);
	SetFilePointer(hFile,dwNewFsize,NULL,FILE_BEGIN);
	SetEndOfFile(hFile);
	CloseHandle(hFile);

	// show the result
	printf("Result:\n");
	printf(" Old filesize: %i\n",dwFsize);
	printf(" New filesize: %i\n",dwNewFsize);
	printf(" Minimization: %i%c\n\n",100 - (100 * dwNewFsize) / dwFsize,'%');

	// THE END
	printf(" DONE...have fun\n");
	return 0;
}
